@@ -283,7 +283,7 @@ def generate_core_dummy(build_cfg, core, unit_cfg):
     LOG.info("Finished generating Core Dummy (in %4.2f s)", time.time() - start_time)
 
 
-def generate_ext_var(build_cfg, unit_cfg, signal_if, udt, asil_level_db, asil_level_dep, debug_code=True):
+def generate_external_var(build_cfg, unit_cfg, udt, asil_level_dep, nrm_dict, dep_dict, sec_dict):
     """Generate two c-files that define the signal interface to the supplier.
 
     The VcExtVar function assigns all variables to the CVC_DISP memory area,
@@ -296,28 +296,13 @@ def generate_ext_var(build_cfg, unit_cfg, signal_if, udt, asil_level_db, asil_le
 
     Args:
         build_cfg (BuildProjConfig): Build project class holding where files should be stored.
-        signal_if (SignalInterfaces): class holding signal interface information.
+        unit_cfg (UnitConfigs) : Aggregated unit configs class.
         udt (UserDefinedTypes): Class holding user defined data types.
-        asil_level_db (str): ASIL level for debug variables.
-        asil_level_dep (str): ASIL level for dependability variables.
-        debug_code (boolean): If true, generate debug code.
+        asil_level_dep (str): ASIL level for the dependability variables.
+        nrm_dict (dict): Dictionary with normal variables.
+        dep_dict (dict): Dictionary with dependability variables.
+        sec_dict (dict): Dictionary with secure variables.
     """
-    LOG.info("******************************************************")
-    LOG.info("Start generating VcExtVar and VcDebug")
-    start_time = time.time()
-    nrm_dict, dep_dict, sec_dict, dbg_dict = signal_if.get_external_io()
-    _extract_external_var(build_cfg, unit_cfg, udt, asil_level_dep, nrm_dict, dep_dict, sec_dict)
-    if debug_code:
-        _extract_debug(build_cfg, unit_cfg, asil_level_db, dep_dict, dbg_dict)
-    LOG.info(
-        "Finished generating VcExtVar and VcDebug (in %4.2f s)",
-        time.time() - start_time,
-    )
-
-
-def _extract_external_var(
-    build_cfg, unit_cfg, udt, asil_level_dep, nrm_dict, dep_dict, sec_dict
-):
     if build_cfg.has_yaml_interface:
         ext_var_nrm = ExtVarYaml(nrm_dict, build_cfg, unit_cfg, udt)
         ext_var_dep = ExtVarYaml(dep_dict, build_cfg, unit_cfg, udt, asil_level_dep)
@@ -339,19 +324,6 @@ def _extract_external_var(
         instance.generate_files(ext_var_path)
 
 
-def _extract_debug(build_cfg, unit_cfg, asil_level_db, dep_dict, dbg_dict):
-    dbg_instances = {
-        ExtDbg(dbg_dict, build_cfg, unit_cfg): ("VcDebug", "VcDebugOutput"),
-        ExtDbg(dep_dict, build_cfg, unit_cfg, asil_level_db): ("VcDebugSafe", "VcDebugOutputSafe")
-    }
-
-    for instance, dir_names in dbg_instances.items():
-        instance.gen_dbg_files(
-            pjoin(build_cfg.get_src_code_dst_dir(), dir_names[0]),
-            pjoin(build_cfg.get_src_code_dst_dir(), dir_names[1]),
-        )
-
-
 def generate_dummy_var(build_cfg, unit_cfg, signal_if, udt):
     """Generate c-file that define the missing signals.
 
@@ -806,14 +778,30 @@ def build(
         if interface:
             interface_report(build_cfg, unit_cfg, signal_if)
 
-        generate_ext_var(
-            build_cfg,
-            unit_cfg,
-            signal_if,
-            udt,
-            build_defs.CVC_ASIL_LEVEL_MAP[code_generation_config["generalAsilLevelDebug"]],
-            build_defs.ASIL_LEVEL_MAP[code_generation_config["generalAsilLevelDependability"]]
-        )
+        LOG.info("******************************************************")
+        LOG.info("Start generating VcExtVar and VcDebug")
+        start_time = time.time()
+        asil_level_db = build_defs.CVC_ASIL_LEVEL_MAP[code_generation_config["generalAsilLevelDebug"]]
+        asil_level_dep = build_defs.ASIL_LEVEL_MAP[code_generation_config["generalAsilLevelDependability"]]
+        nrm_dict, dep_dict, sec_dict, dbg_dict = signal_if.get_external_io()
+
+        generate_external_var(build_cfg, unit_cfg, udt, asil_level_dep, nrm_dict, dep_dict, sec_dict)
+
+        restructured_dbg_dict = {}
+        dbg_instances = {
+            ExtDbg(dbg_dict, build_cfg, unit_cfg): ("VcDebug", "VcDebugOutput"),
+            ExtDbg(dep_dict, build_cfg, unit_cfg, asil_level_db): ("VcDebugSafe", "VcDebugOutputSafe")
+        }
+        for instance, dir_names in dbg_instances.items():
+            restructured_dbg_dict.update({
+                dir_names[0]: instance.dbg_dict["inputs"],
+                dir_names[1]: instance.dbg_dict["outputs"]
+            })
+            instance.gen_dbg_files(
+                pjoin(build_cfg.get_src_code_dst_dir(), dir_names[0]),
+                pjoin(build_cfg.get_src_code_dst_dir(), dir_names[1])
+            )
+        LOG.info("Finished generating VcExtVar and VcDebug (in %4.2f s)", time.time() - start_time)
 
         if not code_generation_config["generateDummyVar"]:
             LOG.info("******************************************************")
@@ -848,19 +836,15 @@ def build(
         LOG.info("******************************************************")
         LOG.info("Start generating the scheduling functions")
         start_time = time.time()
-        gen_schd = SchedFuncs(build_cfg, unit_cfg)
+        gen_schd = SchedFuncs(build_cfg, unit_cfg, restructured_dbg_dict)
         gen_schd.generate_sched_c_fncs(generate_rte_checkpoint_calls)
-        LOG.info(
-            "Finished generating the scheduling functions (in %4.2f s)",
-            time.time() - start_time,
-        )
+        LOG.info("Finished generating the scheduling functions (in %4.2f s)", time.time() - start_time)
+
         LOG.info("******************************************************")
         LOG.info("Start generating the ts header file")
         start_time = time.time()
         gen_schd.generate_ts_defines(pjoin(src_dst_dir, build_cfg.get_ts_header_name()))
-        LOG.info(
-            "Finished generating ts header file (in %4.2f s)", time.time() - start_time
-        )
+        LOG.info("Finished generating ts header file (in %4.2f s)", time.time() - start_time)
 
         # Generate AllSystemInfo.mat for DocGen compatibility
         if generate_system_info:
@@ -888,7 +872,7 @@ def build(
         copy_unit_cfgs_to_output(build_cfg)
         copy_files_to_include(build_cfg)
         if code_generation_config["generateInterfaceHeaders"]:
-            memory_section = MemorySection(build_cfg)
+            memory_section = MemorySection(build_cfg, unit_cfg)
             memory_section.generate_required_header_files()
 
         # Propagate tag name for release builds, TAG_NAME must be set in environment
@@ -917,8 +901,9 @@ def build(
             axis_data = merged_a2l.get_characteristic_axis_data()
             composition_yaml = CompositionYaml(
                 build_cfg,
-                signal_if.composition_spec,
-                unit_cfg, zc_core,
+                signal_if,
+                unit_cfg,
+                zc_core,
                 zc_dids,
                 zc_nvm,
                 axis_data,
@@ -939,16 +924,21 @@ def build(
             if rte_dummy:
                 LOG.info("******************************************************")
                 LOG.info("Generating RTE dummy files")
-                zc_rte = RteDummy(build_cfg, zc_nvm)
+                zc_rte = RteDummy(build_cfg, zc_nvm, composition_yaml.cal_class_info["tl"])
                 zc_rte.generate_rte_dummy()
 
             if code_generation_config["generateCalibrationInterfaceFiles"]:
                 LOG.info("******************************************************")
-                LOG.info("Generating calibration interface files")
-                zc_calibration = ZoneControllerCalibration(
-                    build_cfg, composition_yaml.cal_class_info["tl"]
-                )
-                zc_calibration.generate_calibration_interface_files()
+                if code_generation_config["useCalibrationRteMacroExpansion"]:
+                    LOG.warning(
+                        "Skip generating calibration interface files as useCalibrationRteMacroExpansion is set to true"
+                    )
+                else:
+                    LOG.info("Generating calibration interface files")
+                    zc_calibration = ZoneControllerCalibration(
+                        build_cfg, composition_yaml.cal_class_info["tl"]
+                    )
+                    zc_calibration.generate_calibration_interface_files()
         elif build_cfg.get_ecu_info()[0] == "HI":
             generate_nvm_def(build_cfg, unit_cfg, no_nvm_a2l)
             LOG.info("******************************************************")
