@@ -67,6 +67,7 @@ class BuildProjConfig:
             'generalAsilLevelDebug': 'B',
             'generalAsilLevelDependability': 'B',
             'generateCalibrationInterfaceFiles': False,
+            'useCalibrationRteMacroExpansion': False,
             'generateCoreDummy': False,
             'generateDummyVar': False,
             'generateInterfaceHeaders': False,
@@ -268,7 +269,6 @@ class BuildProjConfig:
             'includeShared': file_config.get('includeShared', True),
             'includeDiagnostics': file_config.get('includeDiagnostics', True),
             'includeNvm': file_config.get('includeNvm', True),
-            'nvmPortPattern': file_config.get('nvmPortPattern', None),
             'scaleMapsAndCurves': file_config.get('scaleMapsAndCurves', True),
         }
         composition_name = file_config.get("compositionName", None)
