@@ -650,11 +650,6 @@ class ZCNVMDef(NVMDef):
 
     def _append_nvm_rte_function_calls(self):
         """Append the NVM RTE function calls to the NVM config source file."""
-        nvm_port_pattern = self._project_config.get_composition_config("nvmPortPattern")
-        if self.valid_nvm_definitions and nvm_port_pattern is None:
-            self.critical("NVM port pattern not defined in ProjectCfg.json. Cannot add NVM RTE function calls.")
-            return
-
         init_function_lines = [
             f'#include "{bd.CVC_CODE_ASIL_D_START}"\n'
             f"{self.init_function} {{\n",
@@ -670,46 +665,38 @@ class ZCNVMDef(NVMDef):
             f'#include "{bd.CVC_CODE_ASIL_D_END}"\n',
         ]
 
-        error_status = []
-        nvsw_init_instructions = False
         main_function_lines = []
 
         for nvm_name, nvm_data in self.valid_nvm_definitions.items():
-            if nvm_data["method"] == "DIRECT-CALL":
-                pim_call = f"Rte_Pim_{nvm_name}()"
-                function_prefix = f"Rte_Call_{nvm_port_pattern.format(NvName=nvm_name)}"
-                main_function_lines.append(f"  *{pim_call} = {nvm_name.lower()};\n")
-                if nvm_data["type"] == "type1":
-                    init_function_lines.append(f"  {function_prefix}_SetRamBlockStatus(TRUE);\n")
-                elif nvm_data["type"] == "type2":
-                    error_status = ["  UInt8 errorStatus = 0;\n"]
-                    main_function_lines.extend([
-                        f"  {function_prefix}_GetErrorStatus(&errorStatus);\n",
-                        "  if (errorStatus == 0) {\n",
-                        f"    {function_prefix}_WriteBlock({pim_call});\n",
-                        "  }\n",
-                    ])
-                else:
-                    self.critical(f"Unknown NVM type: {nvm_data['type']} for NVM: {nvm_name}")
-            elif nvm_data["method"] == "NVSWCOMPONENT":
-                if nvm_data["type"] == "type1":
-                    nvsw_init_instructions = True
+            if nvm_data["type"] not in ["type1", "type2"]:
+                self.critical(f"Unknown NVM type: {nvm_data['type']} for NVM: {nvm_name}")
+                continue
+            if nvm_data["method"] not in ["DIRECT-CALL", "NVSWCOMPONENT"]:
+                self.critical(f"Unknown NVM method: {nvm_data['method']} for NVM: {nvm_name}")
+                continue
+
+            if nvm_data["type"] == "type2":
+                self.warning(
+                    f'{nvm_name.lower()} is a type2 {nvm_data["method"]}. Due to inherent risk type2 write calls are '
+                    'not generated automatically, they have to be written manually and added to source code.'
+                )
+                main_function_lines.append(
+                    f"  // Nvm {nvm_name.lower()} is type 2, add call in manually written source code.\n"
+                )
+            elif nvm_data["type"] == "type1":
+                if nvm_data["method"] == "DIRECT-CALL":
+                    pim_call = f"Rte_Pim_{nvm_name}()"
+                    main_function_lines.append(f"  *{pim_call} = {nvm_name.lower()};\n")
+                    init_function_lines.append(f"  {nvm_name.lower()} = *{pim_call};\n")
+                elif nvm_data["method"] == "NVSWCOMPONENT":
+                    init_function_lines.append(
+                        f"  Rte_Read_{nvm_name.upper()}_{nvm_name.upper()}(&{nvm_name.lower()});\n"
+                    )
                     main_function.append(
-                        f"  (void)Rte_Write_{nvm_name.upper()}_{nvm_name.upper()}(&{nvm_name.lower()});\n"
-                    )
-                elif nvm_data["type"] == "type2":
-                    self.warning(
-                        f'{nvm_name.lower()} is a type2 NVSWCOMPONENT. Due to inherent risk type2 write calls are not '
-                        'generated automatically, they have to be written manually and added to source code.'
-                    )
-                    main_function_lines.append(
-                        f"  // Nvm {nvm_name.lower()} is type 2, add call in manually written source code.\n"
+                        f"  Rte_Write_{nvm_name.upper()}_{nvm_name.upper()}(&{nvm_name.lower()});\n"
                     )
 
-        if nvsw_init_instructions:
-            init_function_lines.append("  // NVSWCOMPONENTS init calls are scheduled by the arxml.\n")
-
-        output = init_function_lines + footer + ["\n"] + main_function + error_status + main_function_lines + footer
+        output = init_function_lines + footer + ["\n"] + main_function + main_function_lines + footer
         with open(self._file_name + ".c", mode="a", encoding="utf-8") as cptr:
             cptr.writelines(output)
 
