@@ -32,6 +32,24 @@ class UserDefinedTypes(ProblemLogger):
             'i8': 'int8_T'
         }
     }
+    SIMULINK_DATA_TYPES = {  # Add more as needed
+        'target_link': {
+            'Simulink.IntEnumType': 'Int32',
+            'int32': 'Int32',
+            'uint16': 'UInt16',
+            'int16': 'Int16',
+            'uint8': 'UInt8',
+            'int8': 'Int8'
+        },
+        'embedded_coder': {
+            'Simulink.IntEnumType': 'int32_T',
+            'int32': 'int32_T',
+            'uint16': 'uint16_T',
+            'int16': 'int16_T',
+            'uint8': 'uint8_T',
+            'int8': 'int8_T'
+        }
+    }
 
     def __init__(self, build_prj_config, unit_configs):
         """Class Initialization.
@@ -49,6 +67,8 @@ class UserDefinedTypes(ProblemLogger):
             )
             raise TypeError(err)
 
+        start_time = time.time()
+        self.info('  Start parsing files with user defined data types')
         self._build_prj_cfg = build_prj_config
         self._unit_configs = unit_configs
         self.enums_per_unit = {}
@@ -57,7 +77,9 @@ class UserDefinedTypes(ProblemLogger):
         self._parse_all_user_defined_types()
         self.common_header_files = []
         # Must run last to be able to compare with TL/EC data types
+        self.all_enums = self._get_enumerations()
         self._interface_data_types = self._parse_interface_data_types()
+        self.info('  Finished parsing files with user defined data types (in %4.2f s)', time.time() - start_time)
 
     @staticmethod
     def convert_interface_enum_to_simulink(interface_enum, underlying_data_type=None):
@@ -111,12 +133,9 @@ class UserDefinedTypes(ProblemLogger):
 
     def _parse_all_user_defined_types(self):
         """Parse all files containing user defined data types."""
-        start_time = time.time()
-        self.info('  Start parsing files with user defined data types')
         src_dirs = self._build_prj_cfg.get_unit_src_dirs()
         for unit, src_dir in src_dirs.items():
             self._parse_unit_user_defined_types(unit, src_dir)
-        self.info('  Finished parsing files with user defined data types (in %4.2f s)', time.time() - start_time)
 
     def _parse_unit_user_defined_types(self, unit, unit_src_dir):
         """Parse unit defined types for a given unit.
@@ -134,7 +153,7 @@ class UserDefinedTypes(ProblemLogger):
         for file_prefix in self.FILE_PREFIXES:
             found_files.extend(Path(unit_src_dir).glob(file_prefix + '*.h'))
         for found_file in found_files:
-            self.info('  Parsing file: %s', str(found_file))
+            self.debug('  Parsing file: %s', str(found_file))
             self._parse_target_link_enum(unit, found_file)
             self._parse_target_link_struct(unit, found_file)
         self._validate_project_enumerations()
@@ -197,27 +216,24 @@ class UserDefinedTypes(ProblemLogger):
             valid_interface_enumerations (dict): Specification, valid interface enumerations.
         """
         valid_interface_enumerations = {}
-        user_defined_enumerations = self.get_enumerations()
-
         for enum_name, enum_data in interface_enumerations.items():
             if enum_name in valid_interface_enumerations:
                 self.critical('%s is multiply defined in interface enumeration definitions.', enum_name)
-            elif enum_name not in user_defined_enumerations:
+            elif enum_name not in self.all_enums:
                 self.critical('%s is not defined in the project', enum_name)
             else:
                 converted = self.convert_interface_enum_to_simulink(
                     enum_data,
-                    user_defined_enumerations[enum_name]['underlying_data_type']
+                    self.all_enums[enum_name]['underlying_data_type']
                 )
                 is_consistent = self._compare_enum_definitions(
-                    user_defined_enumerations[enum_name]['units'],
+                    self.all_enums[enum_name]['units'],
                     enum_name,
                     converted,
-                    user_defined_enumerations[enum_name]
+                    self.all_enums[enum_name]
                 )
                 if is_consistent:
                     valid_interface_enumerations[enum_name] = enum_data
-
         return valid_interface_enumerations
 
     def _validate_project_enumerations(self):
@@ -304,7 +320,11 @@ class UserDefinedTypes(ProblemLogger):
         for enum_name, enum_content in enums:
             enum_members_tmp = re.findall(r'\s*(\w+) = ([-+]?\d+)', enum_content, flags=re.M)
             enum_members = [(k, int(v)) for k, v in enum_members_tmp]
-            underlying_data_type = self._calculate_underlying_data_type(unit, enum_name, enum_members)
+            if enum_name in self.common_enums:
+                underlying_data_type = self.common_enums[enum_name]['underlying_data_type']
+            else:
+                self.warning('Calculating underlying data type for: %s', str(enum_name))
+                underlying_data_type = self._calculate_underlying_data_type(unit, enum_name, enum_members)
             enum_dict = {
                 'underlying_data_type': underlying_data_type,
                 'members': {},
@@ -377,7 +397,6 @@ class UserDefinedTypes(ProblemLogger):
 
         Returns:
             common_enums (dict): Dictionary containing all enums defined in the enum definition directory.
-
         """
         common_enums = {}
         if self._build_prj_cfg.get_enum_def_dir() is None:
@@ -391,9 +410,10 @@ class UserDefinedTypes(ProblemLogger):
             with enum_file.open(mode='r', encoding='ISO-8859-1') as fh:
                 enum_file_content = fh.read()
             enum_name = enum_file.stem
+
             members_part = re.search(
-                r'\s+enumeration\n'
-                r'(\s*\w+\(\d+\)(\s*%.*)?\n|\s*(%.*)?\n)+'
+                r'\s+enumeration\s*\n'
+                r'(\s*%.*|\s*\w+\s*\(\d+\)\s*(%.*)?\n)+'
                 r'\s+end',
                 enum_file_content,
                 flags=re.MULTILINE
@@ -401,8 +421,10 @@ class UserDefinedTypes(ProblemLogger):
             if not members_part:
                 self.warning('Cannot extract enumeration members from %s', str(enum_file))
                 continue
-            members = re.findall(r'(\w+)\((\d+)\)', members_part.group(0))
+            members = re.findall(r'(\w+)\s*\((\d+)\)', members_part.group(0))
             member_dict = {enum_name + '_' + member[0]: int(member[1]) for member in members}
+            enum_members = [(member_name, member_value) for member_name, member_value in member_dict.items()]
+
             get_default_value = re.search(
                 r'function ([A-Za-z0-9]+) = getDefaultValue\(\)[\s\n]*'
                 r'\1 = [A-Za-z0-9]+\.([A-Za-z0-9]+);[\s\n]*'
@@ -412,18 +434,67 @@ class UserDefinedTypes(ProblemLogger):
             )
             if get_default_value is None:
                 self.warning('Cannot extract default enumeration values in: %s', str(enum_file))
-                return None
+                continue
             default_value = get_default_value.groups()[1]
             default_member = f'{enum_name}_{default_value}'
-            enum_members = [(member_name, member_value) for member_name, member_value in member_dict.items()]
-            underlying_data_type = self._calculate_underlying_data_type('target_link', enum_name, enum_members)
+
+            calculate_underlying_data_type = re.search(
+                r'function ([A-Za-z0-9]+) = calculateUnderlyingDataType\(\)[\s\n]*'
+                r'\1 = (true|false);[\s\n]*'
+                r'end',
+                enum_file_content,
+                flags=re.MULTILINE
+            )
+            get_storage_type = re.search(
+                r'classdef[\s]+[A-Za-z0-9]+[\s]*<[\s]*([A-Za-z0-9\.]+)[\s\n]*',
+                enum_file_content,
+                flags=re.MULTILINE
+            )
+            if get_storage_type is None:
+                self.warning('Cannot extract storage type in: %s', str(enum_file))
+                if calculate_underlying_data_type is None or calculate_underlying_data_type.groups()[1] == 'false':
+                    self.warning('Calculating underlying data type for: %s', str(enum_file))
+                    underlying_data_type = self._calculate_underlying_data_type('target_link', enum_name, enum_members)
+                else:
+                    underlying_data_type = self._calculate_underlying_data_type('target_link', enum_name, enum_members)
+            else:
+                underlying_data_type = self.SIMULINK_DATA_TYPES['target_link'][get_storage_type.groups()[0]]
+
             common_enums[enum_name] = {
                 "underlying_data_type": underlying_data_type,
                 "default_value": default_member,
                 "members": member_dict
             }
+
         return common_enums
 
+    def _get_enumerations(self):
+        """Get all enumeration defined in the project, together with unit usage.
+
+        Information already provided in self._validate_project_enumerations during initialization.
+
+        Returns:
+            enumerations (dict): Enumerations defined in the projects, including unit usage.
+        """
+        enumerations = {}
+        for unit, enum_names in self.enums_per_unit.items():
+            for enum_name, enum_data in enum_names.items():
+                if enum_name in enumerations:
+                    enumerations[enum_name]['units'].append(unit)
+                else:
+                    enumerations[enum_name] = deepcopy(enum_data)
+                    enumerations[enum_name]['units'] = [unit]
+        if self._build_prj_cfg.get_code_generation_config("includeAllEnums"):
+            for enum_name, enum_data in self.common_enums.items():
+                if enum_name not in enumerations:
+                    self.warning(
+                        "Enumeration %s is not used in any unit. Included since 'includeAllEnums' is set in config.",
+                        enum_name
+                    )
+                    enumerations[enum_name] = deepcopy(enum_data)
+                    enumerations[enum_name]['units'] = []
+        return enumerations
+
     def get_default_enum_value(self, unit, enum_name):
         """Get default value of given enumeration name by searching in the unit configuration.
 
@@ -466,29 +537,10 @@ class UserDefinedTypes(ProblemLogger):
     def get_enumerations(self):
         """Get all enumeration defined in the project, together with unit usage.
 
-        Information already provided in self._validate_project_enumerations during initialization.
-
         Returns:
-            enumerations (dict): Enumerations defined in the projects, including unit usage.
+            self.all_enums (dict): Enumerations defined in the projects, including unit usage.
         """
-        enumerations = {}
-        for unit, enum_names in self.enums_per_unit.items():
-            for enum_name, enum_data in enum_names.items():
-                if enum_name in enumerations:
-                    enumerations[enum_name]['units'].append(unit)
-                else:
-                    enumerations[enum_name] = deepcopy(enum_data)
-                    enumerations[enum_name]['units'] = [unit]
-        if self._build_prj_cfg.get_code_generation_config("includeAllEnums"):
-            for enum_name, enum_data in self.common_enums.items():
-                if enum_name not in enumerations:
-                    self.warning(
-                        "Enumeration %s is not used in any unit. Included since 'includeAllEnums' is set in config.",
-                        enum_name
-                    )
-                    enumerations[enum_name] = deepcopy(enum_data)
-                    enumerations[enum_name]['units'] = []
-        return enumerations
+        return self.all_enums
 
     def get_interface_data_types(self):
         """Returns all interface data types"""
