@@ -15,12 +15,12 @@ from powertrain_build.zone_controller.calibration import ZoneControllerCalibrati
 class CompositionYaml(ProblemLogger):
     """Class for handling ZoneController composition yaml generation."""
 
-    def __init__(self, build_cfg, composition_spec, unit_cfg, zc_core, zc_dids, zc_nvm, a2l_axis_data, enums):
+    def __init__(self, build_cfg, signal_if, unit_cfg, zc_core, zc_dids, zc_nvm, a2l_axis_data, enums):
         """Init.
 
         Args:
             build_cfg (BuildProjConfig): Object with build configuration settings.
-            composition_spec (dict): Dict with port interface information.
+            signal_if (SignalInterfaces): Class holding signal interface information.
             unit_cfg (UnitConfig): Object with unit configurations.
             zc_core (ZCCore): Object with zone controller diagnositic event information.
             zc_dids (ZCDIDs): Object with zone controller diagnostic DID information.
@@ -40,7 +40,8 @@ class CompositionYaml(ProblemLogger):
         }
         self.build_cfg = build_cfg
         self.unit_src_dirs = build_cfg.get_unit_src_dirs()
-        self.composition_spec = composition_spec
+        self.composition_spec = signal_if.composition_spec
+        self.external_io = signal_if.get_external_io()
         self.unit_cfg = unit_cfg
         self.zc_core = zc_core
         self.zc_dids = zc_dids
@@ -57,7 +58,12 @@ class CompositionYaml(ProblemLogger):
         self.calibration_init_values = self.get_init_values(calibration_variables)
         self.cal_class_info = self._get_class_info(calibration_variables)
         self.meas_class_info = self._get_class_info(measurable_variables)
+        self.include_calibration_interface_files = False
         if self.build_cfg.get_code_generation_config(item="generateCalibrationInterfaceFiles"):
+            self.include_calibration_interface_files = True
+            if self.build_cfg.get_code_generation_config(item="useCalibrationRteMacroExpansion"):
+                self.include_calibration_interface_files = False
+        if self.include_calibration_interface_files:
             trigger_read_rte_cdata_signal_name = self._get_calibration_trigger_signal_name(calibration_variables)
             self.cal_class_info["autosar"]["class_info"].update(
                 {
@@ -82,6 +88,32 @@ class CompositionYaml(ProblemLogger):
             return float(value_str[:-1])
         return int(value_str)
 
+    @staticmethod
+    def _prepare_for_xml(string):
+        """Prepare a string for XML serialization.
+
+        Args:
+            string (str): The string to prepare.
+        Returns:
+            xml_string (str): The prepared string.
+        """
+        limited_string = string[:255]
+        no_line_breaks_string = "".join(limited_string.splitlines())
+        xml_string = scalarstring.DoubleQuotedScalarString(
+            no_line_breaks_string.replace(
+                "&", "&amp;"
+            ).replace(
+                "<", "&lt;"
+            ).replace(
+                ">", "&gt;"
+            ).replace(
+                "'", "&apos;"
+            ).replace(
+                '"', "&quot;"
+            )
+        )
+        return xml_string
+
     def get_base_data_types(self):
         """Create base data types in expected Autosar/yaml2arxml format."""
         base_data_types = {
@@ -121,6 +153,7 @@ class CompositionYaml(ProblemLogger):
             encoding="utf-8",
         ) as file:
             yaml = YAML()
+            yaml.width = 1000  # We don't want to wrap lines in the yaml file
 
             def modify_float_representation(s):
                 return re.sub(r"(\s-?\d+)e(?=\+|-\d)", r"\1.e", s)
@@ -316,14 +349,29 @@ class CompositionYaml(ProblemLogger):
         """
         diag_dict = {}
         diagnostics = self.composition_spec.get("diagnostics", {})
-        dids = diagnostics.get("dids", {})
-        self.zc_dids.valid_dids = dids
-        events = self._edit_event_dict(diagnostics.get("events", {}))
+
+        if self.build_cfg.get_composition_config("includeDiagnostics") == "manual":
+            dids = diagnostics.get("dids", {})
+            events = self._edit_event_dict(diagnostics.get("events", {}))
+        elif self.build_cfg.get_composition_config("includeDiagnostics") == "manual_dids":
+            dids = diagnostics.get("dids", {})
+            events_tmp = self._edit_event_dict(diagnostics.get("events", {}))
+            events = self.zc_core.get_diagnostic_trouble_codes(events_tmp)
+        elif self.build_cfg.get_composition_config("includeDiagnostics") == "manual_dtcs":
+            self.zc_dids.valid_dids = diagnostics.get("dids", {})
+            dids = self.zc_dids.valid_dids
+            events = self._edit_event_dict(diagnostics.get("events", {}))
+        else:
+            self.zc_dids.valid_dids = diagnostics.get("dids", {})
+            dids = self.zc_dids.valid_dids
+            events_tmp = self._edit_event_dict(diagnostics.get("events", {}))
+            events = self.zc_core.get_diagnostic_trouble_codes(events_tmp)
         rids = diagnostics.get("rids", {})
+
         if dids:
-            diag_dict["dids"] = self.zc_dids.valid_dids
+            diag_dict["dids"] = dids
         if events:
-            diag_dict["events"] = self.zc_core.get_diagnostic_trouble_codes(events)
+            diag_dict["events"] = events
         if rids:
             diag_dict["rids"] = rids
             self.warning("Will not generate code for RIDs, add manually.")
@@ -443,25 +491,37 @@ class CompositionYaml(ProblemLogger):
         Returns:
             dict: Dict containing runnables information.
         """
-        swc_name = self.build_cfg.get_composition_config("softwareComponentName")
         autosar_prefix = "AR_"
         swc_prefix = self.build_cfg.get_scheduler_prefix()
         custom_step_function = self.build_cfg.get_composition_config("customYamlStepFunctionName")
         custom_init_function = self.build_cfg.get_composition_config("customYamlInitFunctionName")
         standard_init_function = autosar_prefix + swc_prefix + "VcExtINI"
         init_function = custom_init_function if custom_init_function is not None else standard_init_function
-        call_dict = self._get_runnable_calls_info()
-        mode_switch_points_dict = self.composition_spec.get("mode_switch_points", {})
-        runnables = self.build_cfg.get_units_raster_cfg()["SampleTimes"]
-        calibration_variables = list(self.cal_class_info["autosar"]["class_info"].keys())
+        calibration_variables = list(
+            self.cal_class_info["autosar"]["class_info"].keys()
+        ) + list(
+            self.composition_spec.get("shared", {}).keys()
+        )
         swc_content = {init_function: {"type": "INIT", "accesses": calibration_variables}}
 
-        if self.build_cfg.get_code_generation_config(item="generateCalibrationInterfaceFiles"):
+        if self.include_calibration_interface_files:
+            swc_name = self.build_cfg.get_composition_config("softwareComponentName")
             cal_init_function = autosar_prefix + ZCC.calibration_function_init_template.format(swc_name=swc_name)
             cal_step_function = autosar_prefix + ZCC.calibration_function_step_template.format(swc_name=swc_name)
             swc_content[cal_init_function] = {"type": "INIT", "accesses": calibration_variables}
             swc_content[cal_step_function] = {"type": "PERIODIC", "period": 0.1, "accesses": calibration_variables}
 
+        call_dict = self._get_runnable_calls_info()
+        mode_switch_points_dict = self.composition_spec.get("mode_switch_points", {})
+        reads = []
+        writes = []
+        for port, port_data in self._get_ports_info().items():
+            if port_data["direction"] in ["IN", "CLIENT"]:
+                reads.append(port)
+            else:
+                writes.append(port)
+        runnables = self.build_cfg.get_units_raster_cfg()["SampleTimes"]
+
         if len(runnables) == 1 and custom_step_function is not None:
             swc_content[custom_step_function] = {
                 "type": "PERIODIC",
@@ -472,6 +532,10 @@ class CompositionYaml(ProblemLogger):
                 swc_content[custom_step_function]["calls"] = call_dict
             if mode_switch_points_dict:
                 swc_content[custom_step_function]["mode_switch_points"] = mode_switch_points_dict
+            if reads:
+                swc_content[custom_step_function]["reads"] = reads
+            if writes:
+                swc_content[custom_step_function]["writes"] = writes
             return swc_content
 
         if custom_step_function is not None:
@@ -490,6 +554,10 @@ class CompositionYaml(ProblemLogger):
                 swc_content[key]["calls"] = call_dict
             if mode_switch_points_dict:
                 swc_content[key]["mode_switch_points"] = mode_switch_points_dict
+            if reads:
+                swc_content[key]["reads"] = reads
+            if writes:
+                swc_content[key]["writes"] = writes
 
         return swc_content
 
@@ -511,11 +579,23 @@ class CompositionYaml(ProblemLogger):
         swcs[software_component_name]["runnables"] = self._get_runnable_info()
         if self.build_cfg.get_composition_config("includeShared"):
             swcs[software_component_name]["shared"] = self.cal_class_info["autosar"]["class_info"]
+            for variable_name, variable_info in self.composition_spec.get("shared", {}).items():
+                if variable_name in swcs[software_component_name]["shared"]:
+                    self.critical("Shared variable %s already defined in project.", variable_name)
+                else:
+                    swcs[software_component_name]["shared"][variable_name] = variable_info
         if self.build_cfg.get_composition_config("includeStatic"):
             swcs[software_component_name]["static"] = self.meas_class_info["autosar"]["class_info"]
+            for variable_name, variable_info in self.composition_spec.get("static", {}).items():
+                if variable_name in swcs[software_component_name]["static"]:
+                    self.critical("Static variable %s already defined in project.", variable_name)
+                else:
+                    swcs[software_component_name]["static"][variable_name] = variable_info
         swcs[software_component_name]["ports"] = self._get_ports_info()
+        if self.composition_spec.get("io") is not None:
+            swcs[software_component_name]["io"] = self.composition_spec["io"]
         diagnostic_info = self._get_diagnostic_info()
-        if self.build_cfg.get_composition_config("includeDiagnostics"):
+        if self.build_cfg.get_composition_config("includeDiagnostics") is not False:
             swcs[software_component_name]["diagnostics"] = diagnostic_info
         nvm_info, nvm_data_types_tmp, static_variables = self._get_nvm_info()
         if self.build_cfg.get_composition_config("includeNvm"):
@@ -554,6 +634,12 @@ class CompositionYaml(ProblemLogger):
                         calibration_variables[signal_name] = info
                     elif "CVC_DISP" in info["class"]:
                         measurable_variables[signal_name] = info
+        # External inports should also be considered as measurable variables
+        for io_type in self.external_io:
+            for signal_name in io_type.get("input", {}).keys():
+                for signal_data in config["inports"][signal_name].values():
+                    measurable_variables[signal_name] = signal_data
+                    continue  # Inports can appear in several units, pick first one
         return calibration_variables, measurable_variables
 
     def _get_class_info(self, variable_dict):
@@ -613,9 +699,11 @@ class CompositionYaml(ProblemLogger):
         if not isinstance(info["width"], list):
             class_info[signal_name] = {
                 "type": info["type"],
-                "access": "READ-ONLY" if "CVC_DISP" in info["class"] else "READ-WRITE",
+                "access": "READ-ONLY" if "CVC_DISP" in info["class"] or info["class"] == "CVC_EXT" else "READ-WRITE",
                 "init": self.calibration_init_values.get(signal_name, max(min(0, upper), lower)),
             }
+            if info["description"]:
+                class_info[signal_name]["longname"] = self._prepare_for_xml(info["description"])
             return class_info, data_types
 
         if isinstance(lower, list) or isinstance(upper, list):
@@ -723,5 +811,7 @@ class CompositionYaml(ProblemLogger):
             "access": "READ-ONLY" if "CVC_DISP" in info["class"] else "READ-WRITE",
             "init": init,
         }
+        if info["description"]:
+            class_info[signal_name]["longname"] = self._prepare_for_xml(info["description"])
         data_types = {**data_types, **new_data_type}
         return class_info, data_types
